function [dzdy] = ddy(z,y)

%ddy(z,y): Differentiates matrix z with respect to y using a
% centred difference approximation (uncentred at the edges)
% Use: dzdy=ddy(z,y)
% y can be a matrix, in which case y-values increase down columns

[ny nx]=size(z);
[m n]=size(y);
if m==1
  [xg yg]=meshgrid(1:nx,y);
else
  yg=y;    %don't need xg anyway
end


dy(2:(ny-1),:)=yg(3:ny,:)-yg(1:(ny-2),:);

dy(1,:)=yg(3,:)-yg(1,:);
dy(ny,:)=yg(ny,:)-yg((ny-2),:);

dzdy(2:(ny-1),:)=(z(3:ny,:)-z(1:(ny-2),:))./dy(2:(ny-1),:);

dzdy(1,:)=(4*z(2,:)-3*z(1,:)-z(3,:))./dy(1,:);
dzdy(ny,:)=-(4*z((ny-1),:)-3*z(ny,:)-z((ny-2),:))./dy(1,:);