function [vel]=interpolate2(v,range,elev,xr,zr)

%interpolate2(v,range,elev,xr,zr): Interpolates polar data 
% onto a rectangular grid given by vectors xr and zr.
%Use: vr=interpolate2(v,range,elev,xr,zr)
% where v is the matrix to be interpolated
% range is a monotonically increasing vector of ranges (not
%    necessarily equal spacing)
% elev is like range, in degrees
% xr, zr are monotonically increasing vectors, equally spaced (though 
%    dx does not have to equal dz)
% Values not interpolatable are set to NaN
% Interpolation across velocity folding is taken care of (should not
% affect interpolation of other variables)

vel=NaN;
[mm nn]=size(v);

%sort out grids etc

angles=elev(1:nn)';
X=range'*cos(angles'*pi/180);
Z=range'*sin(angles'*pi/180);

nx=length(nx);
nz=length(zr);

%make sure vel is of correct size
vel(nx,nz)=0;
vel=vel+NaN;

%which way was the scan done??
if angles(nn)>angles(1);
  scandir=+1;
else
  scandir=-1;
end

%start interpolating
clc;
for j=1:nz,
home;
rowsleft=nz-j
  for i=1:nx,
    x=xmin+i*dx;
    z=zmin+j*dz;
    r=(x^2+z^2)^0.5;
    el=atan(z/x)*180/pi;
    
    if el<=max(elev)

    m=find(range<=r);
    m1=max(m);

    n=find(angles<=el);
    n1=(scandir==1)*max(n)+(scandir==-1)*min(n);

    m2=m1+1;
    m3=m1+1;
    m4=m1;
 
    n2=n1;
    n3=n1+scandir;
    n4=n1+scandir;

    if (m1>=1)&(m2<=mm)&(min([n1 n3])>=1)&(max([n1 n3])<=nn) 

    d1=((x-X(m1,n1))^2+(z-Z(m1,n1))^2)^(-0.5);
    d2=((x-X(m2,n2))^2+(z-Z(m2,n2))^2)^(-0.5);
    d3=((x-X(m3,n3))^2+(z-Z(m3,n3))^2)^(-0.5);
    d4=((x-X(m4,n4))^2+(z-Z(m4,n4))^2)^(-0.5);

    vel(i,j)=(d1*v(m1,n1)+d2*v(m2,n2)+d3*v(m3,n3)+d4*v(m4,n4))/(d1+d2+d3+d4);
    vel(i,j)=vel(i,j)/cos(el*pi/180);
    end
    end
  
  end
end







