/*
loadchil.c

*/
#define VERSION "0.1 (9/10/99)"

#include "mex.h"
#include "stdlib.h"
#include "stdio.h"
#include "string.h"

#include "chil.h"
#include "mextools.h"

#define WRITE(a) mexPrintf(a)
#define JUMPSIZE 512
#define TEMPBUFLEN 20
#define NO_SIG_VAL -999.0
//#define MAX_FILES 15

/*
char *field_names[] = { UNKNOWN_NAME , ZH_NAME , ZDR_NAME , 
LDR_NAME , PHIDP_NAME , VEL1_NAME , VEL2_NAME , WIDTH_NAME };
char *Zh2_name=ZH2_NAME;
char *system_names[] = { "unknown","camra","rabelais","galileo","singapore"};
*/

unsigned int verbose=0,quiet=0,progress=0,reqday=0,reqmonth=0,reqyear=0,rangecorrection=0;
int CreateMatlabArray(char * name,double * data_ptr,int m,int n) {
   mxArray *field;
   double *pr_ptr;

   if (verbose) mexPrintf("Writing out matrix: %s\n",name);
   field=mxCreateDoubleMatrix(m,n,mxREAL);
   mxSetName(field,name);
   pr_ptr=mxGetPr(field);
   memcpy(pr_ptr,data_ptr,m*n*sizeof(double));   
   mexPutArray(field,"base");
   mxFree(data_ptr);
   mxDestroyArray(field); 
   return 0;
}
int CreateMatlabScalar(char * name,double value) {
   mxArray *field;

   if (verbose) mexPrintf("Writing out scalar: %s\n",name);

   if ((field=mxCreateDoubleMatrix(1,1,mxREAL)) == NULL) {
      mexWarnMsgTxt("Unable to create scalar\n");
      return 1;
   }
   mxSetName(field,name);
   *mxGetPr(field)=value;
   mexPutArray(field,"base");
   mxDestroyArray(field);
   return 0;
}

unsigned int GetUIntFromMatlabArray(const mxArray * array) {
   char buf[TEMPBUFLEN];
   unsigned int value;
   if (mxIsChar(array)) {
      mxGetString(array,buf,TEMPBUFLEN);
      sscanf(buf,"%u",&value);
   }
   else {
      value=(unsigned int) *mxGetPr(array);
   }
   return value;
}

float GetFloatFromMatlabArray(const mxArray * array) {
   char buf[TEMPBUFLEN];
   float value;
   if (mxIsChar(array)) {
      mxGetString(array,buf,TEMPBUFLEN);
      sscanf(buf,"%f",&value);
   }
   else {
      value=(float) *mxGetPr(array);
   }
   return value;
}

#define MemoryError mexErrMsgTxt("Failed to allocate memory")
void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[]) {

RasterInfo raster_info;
ChilboltonFile chil_file;
FileList file_list;
Ray ray;

int status,i,j,firsttime=1;
unsigned int raster,scan=1,avegates=1,averays=1,process=0,clean=0,file=0,noisegates=25,newrangeoffset=0,basics=0,suppressclutter=0;
char *filebuf = NULL,tempbuf[TEMPBUFLEN];
char defaultsumdir[]="/export/anvil/anvil-01/swrhgnrj/sum";
float noisefactor=3.0,rangeoffset=0.0,Zcal=0.0,sigma_v_floor=0.0,v_floor=0.0;

int buflen,iray=0,iblock=0,nblock=0,datelen;
long int newsize, allocsize;

double *field_ptr[MAX_FIELDS],*elev_ptr=NULL,*azim_ptr=NULL,*utc_ptr=NULL,* range_ptr=NULL,*date_ptr=NULL,*nez_ptr=NULL;
double nan_value;

int argc;
char **argv,*argt;
char errbuf[511];

/* matlab remembers global variables! must clear them */
verbose=0;quiet=0;progress=0;reqday=0;reqmonth=0;reqyear=0;rangecorrection=0;
sigma_v_floor=0.0;v_floor=0.0;

nan_value=mxGetNaN();

CHILSecondarySumDir(defaultsumdir);

/* If too few arguments, give a usage description */
if (nrhs < 2) {
   mexPrintf("This is `loadchil', a mex file for reading Chilbolton radar data into matlab\n");
   mexPrintf("Version %s\n",VERSION);
   mexPrintf("Full documentation at  http://www.met.rdg.ac.uk/~radar/doc/loadchil/\n");
   mexPrintf("Contact Robin Hogan (R.J.Hogan@rdg.ac.uk) with suggestions, queries & free beer\n");
   mexPrintf("Usage:\n   loadchil file1.dat raster1-rasterN [file2.dat raster1-rasterN ...] [options]\n");
   mexPrintf("   loadchil -date YYYYMMDD filepattern1 [filepattern2 ...] [options]\n");
   mexPrintf("Options:\n");
   mexPrintf("   -averays n      average n rays\n");
   mexPrintf("   -avegates n     average n gates\n");
   mexPrintf("   -scan n         get scan n\n");
   mexPrintf("   -verbose        display raster and file information\n");
   mexPrintf("   -quiet          display no information except errors\n");
   mexPrintf("   -basicsonly     don't write those pointless variables like min_gate etc.\n");
   mexPrintf("   -process        subtract noise (Zh must be present) using gates\n");
   mexPrintf("                   from the top of the ray to characterise noise\n");
   mexPrintf("   -oscillator     subtract noise using gates from the top of the ray\n");
   mexPrintf("                     to characterise oscillator interference\n");
   mexPrintf("   -noisegates n   use n gates to characterise noise/interference\n");
   mexPrintf("                     (default %hu)\n",noisegates);
   mexPrintf("   -noisefactor f  set noise-elimination factor to f (default %f)\n",noisefactor);
   mexPrintf("   -clean          remove erroneous lines\n");
   mexPrintf("   -noclutter      suppress clutter using v and sigma_v\n");
   mexPrintf("   -day n          only accept data written on day n\n");
   mexPrintf("   -correctrange   perform range correction\n");
   mexPrintf("   -rangeoffset f  perform range correction with offset of f metres\n");
   mexPrintf("   -Zcal f         add f dB to raw Zh values\n");
   mexPrintf("   -v_floor f      set equivalent noise floor for v to f dB above that for Zh\n");
   mexPrintf("   -sv_floor f     set equiv. noise floor for sigma_v to f dB above that for Zh\n");
   mexPrintf("   -sumdir s       set fallback summary-file directory to s (default\n");
   mexPrintf("                     %s)\n",defaultsumdir);
   return;
}

/* Save a lot of time and effort: convert all string arguments at the start */
argc=nrhs;
MexConvertStringArguments(nrhs,prhs,&argv,&argt);

/* Next, check if the first argument is a switch such as "-date" */
// mxGetString(prhs[0],tempbuf,TEMPBUFLEN-1); 
i=0;
if (strcmp(argv[0],"-date")==0) {
   /* The program must read all summary files to decide what rasters to load */
//   if (mxGetString(prhs[1],tempbuf,TEMPBUFLEN-1) != 0) 
   if (argt[1] != mxCHAR_CLASS) 
      mexErrMsgTxt("The date should be a string argument");
   datelen=strlen(argv[1]);
   if (datelen==6) {
      i=sscanf(argv[1],"%02u%02u%02u",&reqyear,&reqmonth,&reqday);
   }
   else if (datelen==8) {
      i=sscanf(argv[1],"%04u%02u%02u",&reqyear,&reqmonth,&reqday);
      reqyear=reqyear-1900;
   }
   if (i<3)
      mexErrMsgTxt("Can't understand date: should be in the form `yymmdd' or `yyyymmdd'");

   if (nrhs < 3) {
      mexErrMsgTxt("Need at least one filename");
   }

   /* Translate all filename arguments (not beginning with "-") into strings */
/*   j=2;
   filebuf=NULL;
   while (j<nrhs) {
      buflen = (mxGetM(prhs[j]) * mxGetN(prhs[j])) + 1;
      if (filebuf==NULL) 
         if ((filebuf = (char *) mxCalloc(buflen,sizeof(char))) == NULL) MemoryError;
      else
         if ((filebuf = (char *) mxRealloc(filebuf, buflen+ifilename[nfiles])) == NULL) MemoryError; 
      filebuf
   }
*/

   if (CHILGetFileList(reqday,reqmonth,reqyear,argc-2,argv+2,&file_list)<1) {
      mexPrintf("Can't find any data in these files with this date\n");
      return;
   }
/* Find the first switch */
   for (j=3;j<nrhs;j++) {
      if (argt[j] != mxCHAR_CLASS )
         break;
      if (*argv[j]=='-')
         break;
   }
}
else {
/* The first pairs of arguments are filenames and raster ranges.
 * for non-cloud data one would usually expect a single file and raster */
j=0;
file_list.n=0;
while (j<nrhs) {
   /* first the filename */
   if (argt[j] != mxCHAR_CLASS)
      mexErrMsgTxt("Unexpected numeric argument");
   file_list.name[file_list.n]=argv[j];

   if (*file_list.name[file_list.n] == '-')
      /* found a switch: end of file/raster pairs */
      break;
   /* then the raster range */
   if (++j >= nrhs) {
      mexErrMsgTxt("No raster number(s) specified for file");
   }
   if (argt[j]==mxCHAR_CLASS) {
      i=sscanf(argv[j],"%hu-%hu",&file_list.first[file_list.n],&file_list.last[file_list.n]);
      if (i==0) {
         mexErrMsgTxt("Cannot read argument following filename as raster number");
      } else if (i==1) {
         file_list.last[file_list.n]=file_list.first[file_list.n];
      } else if (file_list.last[file_list.n]<file_list.first[file_list.n]) {
         mexWarnMsgTxt("Last raster # is less than first raster #!");
         file_list.last[file_list.n]=file_list.first[file_list.n];
      }
   }
   else {
      file_list.first[file_list.n]=(unsigned short int) *mxGetPr(prhs[j]);
      file_list.last[file_list.n]=file_list.first[file_list.n];
   }
   j++;
   file_list.n++;
}
}
if (j<nrhs) {
/* we have some switches to parse */
for (i=j;i<nrhs;i++) {
   if (argt[i]==mxCHAR_CLASS) {
      if (strcmp(argv[i],"-averays") == 0 && i+1<nrhs) {
         averays=GetUIntFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-verbose") == 0) {
         verbose=1;
      } else if (strcmp(argv[i],"-quiet") == 0) {
         quiet=1;
      } else if (strcmp(argv[i],"-basicsonly") == 0) {
         basics=1;
      } else if (strcmp(argv[i],"-process") == 0) {
         process=1;
      } else if (strcmp(argv[i],"-oscillator") == 0) {
         process=2;
      } else if (strcmp(argv[i],"-noisegates") == 0 && i+1<nrhs) {
         noisegates=GetUIntFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-clean") == 0) {
         clean=1;
      } else if (strcmp(argv[i],"-noclutter") == 0) {
         suppressclutter=1;
      } else if (strcmp(argv[i],"-correctrange") == 0) {
         rangecorrection=1;
      } else if (strcmp(argv[i],"-rangeoffset") == 0 && i+1<nrhs) {
         newrangeoffset=1;
         rangecorrection=1;
         rangeoffset=GetFloatFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-Zcal") == 0 && i+1<nrhs) {
         Zcal=GetFloatFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-sumdir") == 0 && i+1<nrhs) {
         CHILSecondarySumDir(argv[++i]);
      } else if (strcmp(argv[i],"-noisefactor") == 0 && i+1<nrhs) {
         noisefactor=GetFloatFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-v_floor") == 0 && i+1<nrhs) {
         v_floor=GetFloatFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-sv_floor") == 0 && i+1<nrhs) {
         sigma_v_floor=GetFloatFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-avegates") == 0 && i+1<nrhs) {
         avegates=GetUIntFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-scan") == 0 && i+1<nrhs) {
         scan=GetUIntFromMatlabArray(prhs[++i]);
      } else if (strcmp(argv[i],"-day") == 0 && i+1<nrhs) {
         reqday=GetUIntFromMatlabArray(prhs[++i]);
      } else {
//         scan=GetUIntFromMatlabArray(prhs[i]);
         mexPrintf("Warning: Argument `%s' not implemented\n",argv[i]);
      }
   }
   else {
      scan=(unsigned int) *mxGetPr(prhs[i]);
   }
}   

}

MexFreeArguments(argv);

if (file_list.n==0)
   mexErrMsgTxt("No files specified!");

if ((file_list.n>1 || file_list.first[0] < file_list.last[0]) && (!quiet) && (!verbose))
   progress=1;

/* Loop through all the files */

for (file=0;file<file_list.n;file++) {
if ((status=CHILOpen(file_list.name[file],&chil_file)) != START_OF_FILE) {
   sprintf(errbuf,"Failed to read %s: CHILOpen exit code %d",file_list.name[file],status);
   CHILClose(&chil_file);
   mexErrMsgTxt(errbuf);
}
if (chil_file.summary_exists == 0) {
   CHILClose(&chil_file);
   mexErrMsgTxt("Can't find summary file");
}

if (progress)
   mexPrintf("Opened %s\n   reading raster ",file_list.name[file]);

if (verbose)
   CHILPrintFileInformation(&chil_file);

if (verbose)
   fprintf(stderr,"Reading from raster %u to raster %u, scan %u\n",file_list.first[file],file_list.last[file],scan);

for (raster=file_list.first[file];raster<=file_list.last[file];raster++) {

if (progress)
   mexPrintf("%u ",raster);

if (CHILSeekRaster(&chil_file,raster) !=0 ) {
   if (progress) 
      mexPrintf("(not found) ");
   else
      mexPrintf("\nWarning: Raster %u not found in %s\n",raster,file_list.name[file]); 
   continue;
}

if (scan > 1) {
   if (CHILSkipScans(&chil_file,&raster_info,scan-1) != 0) {
      mexPrintf("Warning: Scan %hu not found in raster\n",scan);
      return;
   }
}

status=CHILReadInformationRecord(&chil_file,&raster_info);
if (newrangeoffset)
   raster_info.range_offset=rangeoffset;

if (verbose)
   CHILPrintInformationRecord(&raster_info);

if (firsttime)
   CHILInitRay(&raster_info,avegates,averays,&ray);
else
   if (CHILReInitRay(&raster_info,avegates,averays,&ray) != 0) {
      CHILClose(&chil_file);
      mexErrMsgTxt("Ray re-initialised with different size");
   }
ray.v_floor=v_floor;
ray.sigma_v_floor=sigma_v_floor;

// fprintf(stderr,"Number of gates: %hu\n",ray.ngates);

if (verbose)
   CHILPrintFieldInformation(&ray);

/* if it's the first file and first raster, allocate memory for matlab variables */
if (firsttime) {
   firsttime=0;
   allocsize=JUMPSIZE*ray.ngates;
#ifdef DEBUG
   fprintf(stderr,"Allocating memory for arrays...");
#endif
#ifdef MATLAB
   if ((range_ptr=(double *) mxCalloc(ray.ngates,sizeof(double))) == NULL) MemoryError;
   if ((elev_ptr=(double *) mxCalloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   if ((azim_ptr=(double *) mxCalloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   if ((utc_ptr=(double *) mxCalloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   if ((date_ptr=(double *) mxCalloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   if (process != 0)
      if ((nez_ptr=(double *) mxCalloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   for (i=0;i<ray.ntotal;i++)
      if ((field_ptr[i]=(double *) mxCalloc(JUMPSIZE*ray.ngates,sizeof(double))) == NULL) MemoryError;
#else
   if ((range_ptr=(double *) calloc(ray.ngates,sizeof(double))) == NULL) MemoryError;
   if ((elev_ptr=(double *) calloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   if ((azim_ptr=(double *) calloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   if ((utc_ptr=(double *) calloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   if ((date_ptr=(double *) calloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   if (process != 0)
      if ((nez_ptr=(double *) calloc(JUMPSIZE,sizeof(double))) == NULL) MemoryError;
   for (i=0;i<ray.ntotal;i++)
      if ((field_ptr[i]=(double *) calloc(JUMPSIZE*ray.ngates,sizeof(double))) == NULL) MemoryError;
#endif
#ifdef DEBUG
   fprintf(stderr,"done.\n");
#endif
   for (i=0;i<ray.ngates;i++)
      *(range_ptr+i)=(double) ray.range[i]+0.001*ray.range_offset;
   nblock=1;
   iblock=0;
   iray=0;
}

while(CHILReadRay(&chil_file,&raster_info,&ray) == 0 && !feof(chil_file.file)) {
if (reqday==0 || ray.day == reqday) {

if (process==1) {
   if (CHILSubtractNoise(&ray,noisegates,noisefactor,NO_SIG_VAL) != 0) 
      mexWarnMsgTxt("Attempt to subtract noise failed");
}
else if (process==2) {
   if (CHILSubtractOscillatorNoise(&ray,noisegates,noisefactor,NO_SIG_VAL) != 0) 
      mexWarnMsgTxt("Attempt to subtract oscillator noise failed");
}
//fprintf(stderr,"[%e] ",*(ray.data[0]+100));

if (suppressclutter)
   CHILSuppressClutter(&ray,NO_SIG_VAL);

if (clean)
   CHILCleanRay(&ray,NO_SIG_VAL);

if (rangecorrection)
   CHILRangeCorrection(&raster_info,&ray,NO_SIG_VAL);

//fprintf(stderr,"[%e] ",*(ray.data[0]+100));
//fprintf(stderr,"[] ");

if (Zcal != 0.0) 
   CHILCalibrateZh(&ray,Zcal,NO_SIG_VAL);
//fprintf(stderr,"[%e] ",*(ray.data[0]+100));

#ifdef DEBUG
fprintf(stderr,"%d ",iray);
#endif
*(elev_ptr+iray)=(double) ray.elevation;
*(azim_ptr+iray)=(double) ray.azimuth;
*(utc_ptr+iray)=(double) ray.utc;
*(date_ptr+iray)=(double) 19000000+ray.year*10000+ray.month*100+ray.day;
if (process != 0)
   *(nez_ptr+iray)=(double) ray.nez;

#ifdef DEBUG 
fprintf(stderr,"%d ",iblock);
#endif

/* Store the ray as doubles */
   for (i=0;i<ray.ntotal;i++) {
      for (j=0;j<ray.ngates;j++) {
         if (*(ray.data[i]+j) != NO_SIG_VAL)
            *(field_ptr[i]+j+iray*ray.ngates) = (double) *(ray.data[i]+j);
         else 
            *(field_ptr[i]+j+iray*ray.ngates) = nan_value;
      }
   }
//   fprintf(stderr,"(%e %e) ",*(ray.data[0]+100),(float) (*(field_ptr[0]+100+iray*ray.ngates))); 
   iray++;
   if (++iblock == JUMPSIZE) {
      if (verbose)
         mexPrintf("Increasing storage space for data\n");
      iblock=0;
      nblock++;
      newsize=JUMPSIZE*ray.ngates*nblock;
#ifdef MATLAB
      if ((elev_ptr=(double *) mxRealloc(elev_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
/*         mxFree(elev_ptr);*/
         MemoryError;
      }
      if ((azim_ptr=(double *) mxRealloc(azim_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
/*         mxFree(azim_ptr);*/
         MemoryError;
      }
      if ((utc_ptr=(double *) mxRealloc(utc_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
/*         mxFree(utc_ptr);*/
         MemoryError;
      }
      if ((date_ptr=(double *) mxRealloc(date_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
/*         mxFree(utc_ptr);*/
         MemoryError;
      }
      if (process != 0)
         if ((nez_ptr=(double *) mxRealloc(nez_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
/*         mxFree(utc_ptr);*/
            MemoryError;
         }
#else
      if ((elev_ptr=(double *) realloc(elev_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
         free(elev_ptr);
         MemoryError;
      }
      if ((azim_ptr=(double *) realloc(azim_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
         free(azim_ptr);
         MemoryError;
      }
      if ((utc_ptr=(double *) realloc(utc_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
         free(utc_ptr);
         MemoryError;
      }
      if ((date_ptr=(double *) realloc(date_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
         free(utc_ptr);
         MemoryError;
      }
      if (process != 0)
         if ((nez_ptr=(double *) realloc(nez_ptr,sizeof(double)*JUMPSIZE*nblock)) == NULL ) {
/*         mxFree(utc_ptr);*/
            MemoryError;
         }
#endif
      for (i=0;i<ray.ntotal;i++) {
#ifdef DEBUG
fprintf(stderr,"Re-allocating memory from %ld to %ld doubles\n",allocsize,newsize);
#endif
#ifdef MATLAB
         if ((field_ptr[i]=(double *) mxRealloc(field_ptr[i],sizeof(double)*newsize)) == NULL ) {
/*            mxFree(field_ptr[i]);*/
            MemoryError;
         }
#else
         if ((field_ptr[i]=(double *) realloc(field_ptr[i],sizeof(double)*newsize)) == NULL ) {
            free(field_ptr[i]);
            MemoryError;
         }
#endif
      }
   }
} /* ray */
}
} /* raster */
#ifdef DEBUG 
fprintf(stderr,"Closing file\n");
#endif
if (progress)
   mexPrintf("\n");
CHILClose(&chil_file);
} /* file */

if (iray==0) {
   mexErrMsgTxt("No data on day specified");
}

/* resize all our arrays */
if (verbose) 
   mexPrintf("Resizing arrays downwards: %d %hu\n",iray,ray.ngates);

#ifdef MATLAB
if ((elev_ptr=(double *) mxRealloc(elev_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
if ((azim_ptr=(double *) mxRealloc(azim_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
if ((utc_ptr=(double *) mxRealloc(utc_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
if ((date_ptr=(double *) mxRealloc(date_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
if (process != 0)
   if ((nez_ptr=(double *) mxRealloc(nez_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
for (i=0;i<ray.ntotal;i++) {
   if ((field_ptr[i]=(double *) mxRealloc(field_ptr[i],sizeof(double)*iray*ray.ngates)) == NULL ) MemoryError;
}
//mxFree(filebuf);
#else
if ((elev_ptr=(double *) realloc(elev_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
if ((azim_ptr=(double *) realloc(azim_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
if ((utc_ptr=(double *) realloc(utc_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
if ((date_ptr=(double *) realloc(date_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
if (process != 0)
   if ((nez_ptr=(double *) realloc(nez_ptr,sizeof(double)*iray)) == NULL ) MemoryError;
for (i=0;i<ray.ntotal;i++) {
   fprintf(stderr,"%d ",i);
   if ((field_ptr[i]=(double *) realloc(field_ptr[i],sizeof(double)*iray*ray.ngates)) == NULL ) MemoryError;
}
free(filebuf);
#endif


if (iray==0) {
   mexWarnMsgTxt("No appropriate data found");
   return;
}

#ifdef DEBUG 
fprintf(stderr,"Writing vectors to matlab workspace\n");
#endif
CreateMatlabArray("range",range_ptr,1,ray.ngates);
CreateMatlabArray("elev",elev_ptr,1,iray);
CreateMatlabArray("azim",azim_ptr,1,iray);
CreateMatlabArray("utc",utc_ptr,1,iray);
CreateMatlabArray("full_date",date_ptr,1,iray);
if (process != 0)
   CreateMatlabArray("nez",nez_ptr,1,iray);

#ifdef DEBUG 
fprintf(stderr,"Writing arrays to matlab workspace\n");
#endif
for (i=0;i<ray.ntotal;i++) {
   CHILFieldName(&ray,i,tempbuf);
   CreateMatlabArray(tempbuf,field_ptr[i],ray.ngates,iray);
}

#ifdef DEBUG 
fprintf(stderr,"Writing scalars to matlab workspace\n");
#endif

if (!basics) {
CreateMatlabScalar("day",(double) raster_info.day);
CreateMatlabScalar("month",(double) raster_info.month);
CreateMatlabScalar("year",(double) raster_info.year);
CreateMatlabScalar("file",(double) raster_info.file);
CreateMatlabScalar("raster",(double) raster_info.raster);
CreateMatlabScalar("start_hour",(double) raster_info.hour);
CreateMatlabScalar("start_min",(double) raster_info.minute);
CreateMatlabScalar("start_sec",(double) raster_info.second);
CreateMatlabScalar("min_gate",(double) raster_info.min_gate);
CreateMatlabScalar("max_gate",(double) raster_info.max_gate);
}
CreateMatlabScalar("system",(double) raster_info.system_id);
CreateMatlabScalar("sweepm",(double) raster_info.sweep_mode);
CreateMatlabScalar("freq",(double) raster_info.frequency);
CreateMatlabScalar("drange",(double) ray.avegates * raster_info.gate_spacing / 1000.0);
CreateMatlabScalar("range_offset",(double) ray.range_offset / 1000.0);

#ifdef DEBUG 
fprintf(stderr,"Freeing ray\n");
#endif
CHILFreeRay(&ray);
#ifdef DEBUG 
fprintf(stderr,"Done\n");
#endif
if (verbose) 
   mexPrintf("Done\n");
}

