#include "mex.h"
/*

	Calculates shear over a specified interval - very slow as it uses a
	linear fit at each location.

	Danny Chapman 24/3/2000

 */

void calcsheard(double *shear, double *vu, double *Z, double dz, int rows, int cols);
void fit(double x[], double y[], int ndata, double *b);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *shear, *vu, *Z;
	double dz;
	int rows, cols;

	/* Check arguments */
	if (nrhs!=3)
		mexErrMsgTxt("Three inputs required");
/* 	if (nlhs!=1) */
/* 		mexErrMsgTxt("One output required"); */
	
	/* Create a pointer to the input matrices, vu and Z */
	vu=mxGetPr(prhs[0]);
	Z=mxGetPr(prhs[1]);

	rows=mxGetM(prhs[0]);
	cols=mxGetN(prhs[0]);

	/* Get the scalar input, dz */
	dz=mxGetScalar(prhs[2]);
	
	/* Set the ouput pointer to the output matrix */
	plhs[0]=mxCreateDoubleMatrix(rows,cols, mxREAL);

	/* Create a pointer to a copy of the ouput matrix*/
	shear=mxGetPr(plhs[0]);

	/* Call the shear subroutine */
	calcsheard(shear,vu,Z,dz,rows,cols);
}

double interp(double x1, double x2, double x, double y1, double y2)
{
	return (y1+ (x-x1)*(y2-y1)/(x2-x1));
}

void calcsheard(double *shear, double *vu_in, double *Z_in, double dz, int rows, int cols)
{
	int i, j, ii, jj, jtop, jbot;
	double *temp1, *temp2, *temp3, *slope, *offset, *vu, *Z;
	mxArray *lhs[2], *rhs[3], *vup[1], *Zp[1];
	int flip=0;
	double thisz, ztop, vtop, zbot, vbot, dv;

	rhs[0]=mxCreateDoubleMatrix(1,cols, mxREAL);
	temp1=mxGetPr(rhs[0]); /* z vals */
	rhs[1]=mxCreateDoubleMatrix(1,cols, mxREAL);
	temp2=mxGetPr(rhs[1]); /* velocities */
	rhs[2]=mxCreateDoubleMatrix(1,1, mxREAL);
	temp3=mxGetPr(rhs[2]); /* degree of fit */
	temp3[0]=1;

	lhs[0]=mxCreateDoubleMatrix(1,1, mxREAL);
	slope=mxGetPr(lhs[0]); /* slope of fit */
	lhs[1]=mxCreateDoubleMatrix(1,1, mxREAL);
	offset=mxGetPr(lhs[1]); /* offset of fit */

	vup[0]=mxCreateDoubleMatrix(rows,cols, mxREAL);
	vu=mxGetPr(vup[0]);
	Zp[0]=mxCreateDoubleMatrix(rows,cols, mxREAL);
	Z=mxGetPr(Zp[0]);

	/* vu must be already unfolded and zapped */

/*  	printf("rows = %d, cols = %d \n", rows, cols);  */

	if (Z_in[0+0*rows] > Z_in[0+(cols-1)*rows]) {
		flip=1;		
		/* flip vu and z*/
		for (i=0; i<rows; i++) {
			for (j=0; j<cols; j++) {
				vu[i+j*rows]=vu_in[i+(cols-j-1)*rows];
				Z[i+j*rows]=Z_in[i+(cols-j-1)*rows];
			}
		}
	} else { /* don't flip */
		for (i=0; i<rows; i++) {
			for (j=0; j<cols; j++) {
				vu[i+j*rows]=vu_in[i+j*rows];
				Z[i+j*rows]=Z_in[i+j*rows];
			}
		}
	}

	/* Start properly */

	for (i=0; i<rows; i++) {
/* 		printf("%d ",i); */
		for (j=0; j<cols; j++) {
			/*find the points dz/2 above and below the current point */
			thisz=Z[i+j*rows];
			/*search upwards...*/
			ztop=mxGetNaN();
			vtop=mxGetNaN();
			jtop=-1; jbot=-1;
			for (jj=j; jj<cols; jj++) {
				if (Z[i+jj*rows] > (thisz+0.5*dz) ) {
/* 					ztop=Z[i+jj*rows]; */
/* 					vtop=vu[i+jj*rows]; */
/* 					vtop=interp(Z[i+(jj-1)*rows],ztop,thisz+0.5*dz,vu[i+(jj-1)*rows],vu[i+jj*rows]); */
/* 					ztop=thisz+0.5*dz; */
					jtop=jj;
					break;
				}
			}
			/*search downwards...*/
			zbot=mxGetNaN();
			vbot=mxGetNaN();
			for (jj=j; jj>=0; jj--) {
				if (Z[i+jj*rows] < (thisz-0.5*dz) ) {
/* 					zbot=Z[i+jj*rows]; */
/* 					vbot=vu[i+jj*rows]; */
/* 					vbot=interp(Z[i+(jj+1)*rows],zbot,thisz-0.5*dz,vu[i+(jj+1)*rows],vu[i+jj*rows]); */
/* 					zbot=thisz-0.5*dz; */
					jbot=jj;
					break;
				}
			}
			/* this just uses the difference from top to bottom...*/
			/* shear[i+j*rows]=(vtop-vbot)/(ztop-zbot); */
			/* fit a straight line between top and bottom */
			if ((jtop != -1) && (jbot != -1)) {
				for (jj=jbot; jj<=jtop ; jj++) {
					temp2[jj-jbot]=vu[i+jj*rows];
					temp1[jj-jbot]=Z[i+jj*rows];
				}
				mxSetN(rhs[0],1+jtop-jbot);
				mxSetN(rhs[1],1+jtop-jbot);
				/* POLYFIT(X,Y,N) finds the coefficients of a polynomial P(X)
					 of degree N that fits the data, P(X(I))~=Y(I), in a
					 least-squares sense. */
/* 				mexCallMATLAB(2,lhs,3,rhs,"polyfit"); */
/* 				slope=mxGetPr(lhs[0]);  */
/* slope of fit */
/* 				printf("slope=%g\t",slope[0]);  */
				fit(temp1,temp2,1+jtop-jbot,slope);
/* 				printf("slope2=%g\n",slope[0]);  */
				shear[i+j*rows]=slope[0];

/* 				if ((i+j*rows) == (40+(cols-90)*rows)) */
/* 					shear[i+j*rows]=-1000; */
				
			} else
				shear[i+j*rows]=mxGetNaN();
		}
	}

	/*flip shear if necessary, use vu as a temporary array*/
	if (Z_in[0+0*rows] > Z_in[0+(cols-1)*rows]) {
		flip=1;		
		/* flip shear and store in vu */
		for (i=0; i<rows; i++) {
			for (j=0; j<cols; j++) {
				vu[i+j*rows]=shear[i+(cols-j-1)*rows];
			}
		}
		/*...and copy the result back into shear */
		for (i=0; i<rows; i++) {
			for (j=0; j<cols; j++) {
				shear[i+j*rows]=vu[i+j*rows];
			}
		}
	}

	mxDestroyArray(lhs[0]);
	mxDestroyArray(lhs[1]);
	mxDestroyArray(rhs[0]);
	mxDestroyArray(rhs[1]);
	mxDestroyArray(rhs[2]);
	mxDestroyArray(vup[0]);
	mxDestroyArray(Zp[0]);

}

/* Given a set of data points x[1..ndata],y[1..ndata] with individual
   standard deviations sig[1..ndata], t them to a straight line y = a
   + bx by minimizing 2 . Returned are a,b and their respective
   probable uncertainties siga and sigb, the chi-square chi2, andthe
   goodness-of- t probability q (that the t would have 2 this large or
   larger). If mwt=0 on input, then the standard deviations are
   assumed to be unavailable: q is returned as 1.0 and the
   normalization of chi2 is to unit standard deviation on all
   points. */

static double sqrarg; 
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg) 

#include <math.h> 
void fit(double x[], double y[], int ndata, double *b) 
{
	/*	double gammq(double a, double x); */
	int i; 
	double wt,t,sxoss,sx=0.0,sy=0.0,st2=0.0,ss,sigdat; 
	*b=0.0; 
	for (i=0;i<ndata;i++) { 
		sx += x[i]; sy +=	y[i]; 
	} 
	ss=ndata; 
	sxoss=sx/ss;
	for (i=0;i<ndata;i++) { 
		t=x[i]-sxoss; 
		st2 += t*t;
		*b += t*y[i];
	}
	*b /= st2;
	/*	*a=(sy-sx*(*b))/ss;*/
}


